"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PathResolver = void 0;
const path_to_regexp_1 = require("path-to-regexp");
class PathResolver {
    constructor(routes) {
        this.routes = [];
        routes.forEach(r => {
            const route = Object.assign({}, r); // clone
            if (route.path[0] === '/')
                throw errorLeadSlash(route);
            // Step 1. All root paths get a prefix '/'
            route.path = '/' + route.path;
            route.redirectTo = init.redirectTo(route.redirectTo, '/');
            // Step 2. All children paths are obtained as follows:
            //         [full parent path] + '/' + [child path]
            route.children = init.children(route);
            this.routes.push(route);
        });
    }
    resolve(pathname) {
        return this.find(pathname, this.routes);
    }
    find(pathname, routes) {
        if (!routes)
            return;
        for (let i = 0; i < routes.length; i++) {
            const route = Object.assign({}, routes[i]);
            const matching = path_to_regexp_1.match(route.path)(pathname);
            if (matching) {
                const pathParams = matching.params;
                if (route.redirectTo) {
                    route.redirectTo = path_to_regexp_1.compile(route.redirectTo)(pathParams);
                }
                route.path = pathname;
                return { route, pathParams };
            }
            const found = this.find(pathname, route.children);
            if (found)
                return found;
        }
    }
}
exports.PathResolver = PathResolver;
const init = {
    redirectTo: (redirectTo, parentPath) => {
        if (typeof redirectTo === 'string') {
            if (redirectTo === '')
                return parentPath;
            else if (redirectTo[0] === '/')
                return redirectTo;
            else {
                const prefix = parentPath === '/' ? '/' : parentPath + '/';
                return prefix + redirectTo;
            }
        }
    },
    children: function ({ path: parentPath, children: routes }) {
        if (!routes)
            return;
        const children = [];
        for (let i = 0; i < routes.length; i++) {
            const route = Object.assign({}, routes[i]); // clone
            const { path } = route;
            if (path[0] === '/')
                throw errorLeadSlash(route);
            children.push(route);
            if (path === '') {
                route.path = parentPath;
            }
            else {
                const prefix = parentPath === '/' ? '' : parentPath;
                route.path = prefix + '/' + path;
            }
            route.redirectTo = this.redirectTo(route.redirectTo, parentPath);
            route.children = this.children(route);
        }
        return children;
    },
};
const errorLeadSlash = (route) => new Error(`Invalid configuration of route '${route.path}': path cannot start with a slash`);
